#!/bin/perl -w
#
# perl script to identify the language of a file based on 
# NEdit's criteria
#
# Permission is granted to distribute this document/source,
# in part or in full, via electronic means or printed copy
# providing that
# (1) that all credits and copyright notices be retained,
# (2) that no charges beyond reproduction be involved, and
# (3) that a reasonable attempt be made to use the most
#     current version available.
#
# revision history:
# date		author/comment
# 1999/02/03	Joor Loohuis (loohuis@geo.uu.nl)
#		original version (derived from other tools)
#
# DATA STRUCTURES:
# language: hash table containing language recognition criteria
#
# SUBS:
# get_language_hash
#   description: read .nedit and extract language recognition
#   	criteria
#   returns: language recognition hash
#   requires: $HOME/.nedit to exist
#
# determine_language
#   description: determine the language of a specified file
#   parameters: file name
#   returns: language name (hash index string), or 
#   	$default_language if no language recognition
#   	criteria match
#   requires: get_language_hash as initialization
#
# NOTES:
# actually, the recognition might not yield the same results as NEdit,
# since the language hash is not traversed in the order given in .nedit
# (I think). This only becomes a problem when conflicting recognition
# criteria exist, but yeah, it's a bug.

# nedit config file, containing language definitions
$nedit_file = "$ENV{'HOME'}/.nedit";

# number of chars to check for regex in file
$test_chars = 200;

# default language mode
$default_language = 'Plain';

# set up language recognition hash table
%language = &get_language_hash;


# simple example: print the language of a file to stdout
$file = shift or die "usage:\n$0 filename\n\n";
$lang = &determine_language($file);
print "$0: language is $lang\n";

# you'll probably want to call nc here, with some cmd line args
# use exec to hand pid over to editor, eg.
# exec "$EDITOR $CMDLINE $file"
#   or die "$EDITOR blew a fuse: $!";


# read .nedit and extract language recognition criteria
sub get_language_hash {
    # slurp in the nedit configuration 
    open nedit_handle, $nedit_file
	or die "$0: $nedit_file not found, $!\n";
    @nedit_config = <nedit_handle>;
    close nedit_handle;

    # skip through to language modes
    do {
	$line = shift @nedit_config;
    } until $line =~ /nedit\.languageModes/;

    # snip off field specifier
    $line =~ s/nedit\.languageModes:\s*//;
    chop $line;
    push @language_mode, $line;

    # go through list of language modes
    do  {
	$line = shift @nedit_config;
	push @language_mode, $line;
    } while ($line =~ /.*\\[ \t]*$/);

    # now get the important data from the definitions
    foreach (@language_mode)  {
	# trim leading garbage from the name
	s/^\s*//;
	($name, $ext, $regex) = split /:/;

	# clean up regex quotes and double backslashes
	if ($regex) {
	    $regex = substr($regex, 1, length($regex) - 2);
	    $regex =~ s/\\\\/\\/g;
	}

	# create a hash of recognition criteria
	$language_hash{$name} = [$ext, $regex];
    }
    return %language_hash;
}


# determine the language of a specified file
sub determine_language {
    $_ = shift;

    # check if the file exists
    if (-e) {
	# yes, read in first $test_chars
	open tmp_handle, $_;
	read tmp_handle, $data, $test_chars;
	close tmp_handle;
    }

    # loop over languages until recognized
    while (($lang, $specs) = each(%language)) {
	# if regex specified, test it
	if (($specs->[1]) and ($data =~ /$specs->[1]/)) {
	    return $lang;
	}

	# otherwize, test for extensions
	@extensions = split /\s/, $specs->[0];
	foreach $ext (@extensions) {
	    $len = length() - length($ext);
	    if ((index($_, $ext) == $len) && ($len >= 0)) {
		return $lang;
	    }
	}
    }

    # not matching any language
    return $default_language;
}
